@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool  Copyright Tobias Oetiker, 1997, 1998, 1999
+ * RRDtool 1.0.24  Copyright Tobias Oetiker, 1997, 1998, 1999
  *****************************************************************************
  * rrd_format.h  RRD Database Format header
  *****************************************************************************/
@@ -7,6 +7,8 @@
 #ifndef _RRD_FORMAT_H
 #define _RRD_FORMAT_H
 
+#include "rrd.h"
+
 /*****************************************************************************
  * put this in your /usr/lib/magic file (/etc/magic on HPUX)
  *
@@ -20,7 +22,7 @@
 #define RRD_VERSION   "0001"
 #define FLOAT_COOKIE  8.642135E130
 
-#if defined(WIN32) || defined(_HPUX_SOURCE)
+#if defined(WIN32)
 #define DNAN          ((double)fmod(0.0,0.0))    
 #define DINF	      ((double)log(0.0))
 #else
@@ -34,10 +36,6 @@
 					       * */
 #endif
 
-
-typedef double       rrd_value_t;         /* the data storage type is
-					   * double */
-
 typedef union unival { 
     unsigned long u_cnt; 
     rrd_value_t   u_val;
@@ -150,7 +148,9 @@ enum ds_param_en {   DS_mrhb_cnt=0,       /* minimum required heartbeat. A
 					   * both can be set to UNKNOWN if you
 					   * do not care. Data outside the limits
  					   * set to UNKNOWN */
-#define DS_NAM_FMT    "%19[a-zA-Z0-9_]"
+
+/* The magic number here is one less than DS_NAM_SIZE */
+#define DS_NAM_FMT    "%19[a-zA-Z0-9_-]"
 #define DS_NAM_SIZE   20
 
 #define DST_FMT    "%19[A-Z]"
