@@ -1,5 +1,5 @@
 /****************************************************************************
- * RRDtool  Copyright Tobias Oetiker, 1997,1998, 1999
+ * RRDtool 1.0.24  Copyright Tobias Oetiker, 1997 - 2000
  ****************************************************************************
  * rrd__graph.c  make creates ne rrds
  ****************************************************************************/
@@ -30,11 +30,12 @@ enum grc_en {GRC_CANVAS=0,GRC_BACK,GRC_SHADEA,GRC_SHADEB,
 enum gf_en {GF_PRINT=0,GF_GPRINT,GF_COMMENT,GF_HRULE,GF_VRULE,GF_LINE1,
 	    GF_LINE2,GF_LINE3,GF_AREA,GF_STACK, GF_DEF, GF_CDEF };
 
-enum op_en {OP_NUMBER=0,OP_VARIABLE,OP_INF,OP_NEGINF,
+enum op_en {OP_NUMBER=0,OP_VARIABLE,OP_INF,OP_PREV,OP_NEGINF,
 	    OP_UNKN,OP_NOW,OP_TIME,OP_ADD,OP_MOD,
             OP_SUB,OP_MUL,
 	    OP_DIV,OP_SIN, OP_DUP, OP_EXC, OP_POP,
 	    OP_COS,OP_LOG,OP_EXP,OP_LT,OP_LE,OP_GT,OP_GE,OP_EQ,OP_IF,
+	    OP_MIN,OP_MAX,OP_LIMIT, OP_FLOOR, OP_CEIL,
 	    OP_UN,OP_END};
 
 enum if_en {IF_GIF=0,IF_PNG=1};
@@ -173,14 +174,15 @@ typedef  struct graph_desc_t {
 
 } graph_desc_t;
 
-#define ALTYGRID      0x01  /* use alternative y grid algorithm */
-#define ALTAUTOSCALE  0x02  /* use alternative algorithm to find lower and upper bounds */
+#define ALTYGRID          0x01  /* use alternative y grid algorithm */
+#define ALTAUTOSCALE      0x02  /* use alternative algorithm to find lower and upper bounds */
+#define ALTAUTOSCALE_MAX  0x04  /* use alternative algorithm to find upper bounds */
 
 typedef struct image_desc_t {
 
     /* configuration of graph */
 
-    char           graphfile[255]; /* filename for graphic */
+    char           graphfile[MAXPATH]; /* filename for graphic */
     long           xsize,ysize;    /* graph area size in pixels */
     col_trip_t     graph_col[__GRC_END__]; /* real colors for the graph */   
     char           ylegend[200];   /* legend along the yaxis */
@@ -219,8 +221,41 @@ typedef struct image_desc_t {
 
 } image_desc_t;
 
-
-
+/* Prototypes */
+int xtr(image_desc_t *,time_t);
+int ytr(image_desc_t *, double);
+enum gf_en gf_conv(char *);
+enum if_en if_conv(char *);
+enum tmt_en tmt_conv(char *);
+enum grc_en grc_conv(char *);
+int im_free(image_desc_t *);
+void auto_scale( image_desc_t *,  double *, char **, double *);
+void si_unit( image_desc_t *);
+void expand_range(image_desc_t *);
+void reduce_data( enum cf_en,  unsigned long,  time_t *, time_t *,  unsigned long *,  unsigned long *,  rrd_value_t **);
+int data_fetch( image_desc_t *);
+long find_var(image_desc_t *, char *);
+long lcd(long *);
+int data_calc( image_desc_t *);
+int data_proc( image_desc_t *);
+time_t find_first_time( time_t,  enum tmt_en,  long);
+time_t find_next_time( time_t,  enum tmt_en,  long);
+void gator( gdImagePtr, int, int);
+int print_calc(image_desc_t *, char ***);
+int leg_place(image_desc_t *);
+int horizontal_grid(gdImagePtr, image_desc_t *);
+int horizontal_log_grid(gdImagePtr, image_desc_t *);
+void vertical_grid( gdImagePtr, image_desc_t *);
+void axis_paint( image_desc_t *, gdImagePtr);
+void grid_paint( image_desc_t *, gdImagePtr);
+gdImagePtr MkLineBrush(image_desc_t *,long, enum gf_en);
+int lazy_check(image_desc_t *);
+int graph_paint(image_desc_t *, char ***);
+int gdes_alloc(image_desc_t *);
+int scan_for_col(char *, int, char *);
+int rrd_graph(int, char **, char ***, int *, int *);
+int bad_format(char *);
+rpnp_t * str2rpn(image_desc_t *,char *);
 
 /* translate time values into x coordinates */   
 /*#define xtr(x) (int)((double)im->xorigin \
@@ -228,14 +263,14 @@ typedef struct image_desc_t {
 		* ((double)(x) - im->start)+0.5) */
 /* initialize with xtr(im,0); */
 int
-xtr(image_desc_t *im,time_t time){
+xtr(image_desc_t *im,time_t mytime){
     static double pixie;
-    if (time==0){
+    if (mytime==0){
 	pixie = (double) im->xsize / (double)(im->end - im->start);
 	return im->xorigin;
     }
     return (int)((double)im->xorigin 
-		 + pixie * ( time - im->start ) );
+		 + pixie * ( mytime - im->start ) );
 }
 
 /* translate data values into y coordinates */
@@ -395,8 +430,8 @@ auto_scale(
 	index = 0;
 	*magfact = 1.0;
     } else {
-	index = floor(log(fabs(*value))/log(im->base)); 
-	*magfact = pow(im->base, index);
+	index = floor(log(fabs(*value))/log((double)im->base)); 
+	*magfact = pow((double)im->base, (double)index);
 	(*value) /= (*magfact);
     }
     if ( index <= symbcenter && index >= -symbcenter) {
@@ -432,8 +467,8 @@ si_unit(
     int   symbcenter = 6;
     double digits;  
     
-    digits = floor( log( max( fabs(im->minval),fabs(im->maxval)))/log(im->base)); 
-    im->magfact = pow(im->base , digits);
+    digits = floor( log( max( fabs(im->minval),fabs(im->maxval)))/log((double)im->base)); 
+    im->magfact = pow((double)im->base , digits);
 #ifdef DEBUG
     printf("digits %6.3f  im->magfact %6.3f\n",digits,im->magfact);
 #endif
@@ -464,10 +499,10 @@ expand_range(image_desc_t *im)
     
 
     
-    #ifdef DEBUG
+#ifdef DEBUG
     printf("Min: %6.2f Max: %6.2f MagFactor: %6.2f\n",
 	   im->minval,im->maxval,im->magfact);
-    #endif
+#endif
 
     if (isnan(im->ygridstep)){
 	if(im->extra_flags & ALTAUTOSCALE) {
@@ -478,6 +513,13 @@ expand_range(image_desc_t *im)
 	      im->minval -= adj;
 	      im->maxval += adj;
 	}
+	else if(im->extra_flags & ALTAUTOSCALE_MAX) {
+	    /* measure the amplitude of the function. Make sure that
+	       graph boundaries are slightly higher than max vals
+	       so we can see amplitude on the graph */
+	      adj = (im->maxval - im->minval) * 0.1;
+	      im->maxval += adj;
+	}
 	else {
 	    scaled_min = im->minval / im->magfact;
 	    scaled_max = im->maxval / im->magfact;
@@ -559,25 +601,30 @@ reduce_data(
         
     for (src_row = skiprows; src_row < row_cnt; src_row+=reduce_factor) {
 	for (col=0;col<(*ds_cnt);col++){
-	    double newval=0;
+	    double newval=DNAN;
 	    unsigned long validval=0;
 	    for (i=0;i<reduce_factor && src_row+i<row_cnt;i++) {		
 		unsigned long ptr = (src_row+i)* (*ds_cnt)+col;
 		if (isnan((*data)[ptr])) continue; /* we can't help with NAN */
 		validval++;
-		switch (cf) {
-		case CF_AVERAGE:
-		    newval += (*data)[ptr];
-		    break;
-		case CF_MINIMUM:
-		    newval = min (newval,(*data)[ptr]);
-		    break;
-		case CF_MAXIMUM:
-		    newval = max (newval,(*data)[ptr]);
-		    break;
-		case CF_LAST:
+		if (isnan(newval)) {
 		    newval = (*data)[ptr];
-		    break;
+		} else {
+		   
+  		  switch (cf) {
+		  case CF_AVERAGE:
+		      newval += (*data)[ptr];
+		      break;
+		  case CF_MINIMUM:
+		      newval = min (newval,(*data)[ptr]);
+		      break;
+		  case CF_MAXIMUM:
+		      newval = max (newval,(*data)[ptr]);
+		      break;
+		  case CF_LAST:
+		      newval = (*data)[ptr];
+		      break;
+		  }
 		}
 	    }
 	    if (validval == 0){newval = DNAN;} else{
@@ -735,8 +782,8 @@ str2rpn(image_desc_t *im,char *expr){
 	    return NULL;
 	}
 
-	else if(sscanf(expr,"%lf%n",&rpnp[steps].val,&pos) == 1){
-	    rpnp[steps].op = OP_NUMBER;
+	else if((sscanf(expr,"%lf%n",&rpnp[steps].val,&pos) == 1) && (expr[pos] == ',')){
+ 	    rpnp[steps].op = OP_NUMBER;
 	    expr+=pos;
 	} 
 	
@@ -754,6 +801,8 @@ str2rpn(image_desc_t *im,char *expr){
 	match_op(OP_SIN,SIN)
 	match_op(OP_COS,COS)
 	match_op(OP_LOG,LOG)
+	match_op(OP_FLOOR,FLOOR)
+	match_op(OP_CEIL,CEIL)
 	match_op(OP_EXP,EXP)
 	match_op(OP_DUP,DUP)
 	match_op(OP_EXC,EXC)
@@ -764,10 +813,14 @@ str2rpn(image_desc_t *im,char *expr){
 	match_op(OP_GE,GE)
 	match_op(OP_EQ,EQ)
 	match_op(OP_IF,IF)
+	match_op(OP_MIN,MIN)
+	match_op(OP_MAX,MAX)
+	match_op(OP_LIMIT,LIMIT)
 	  /* order is important here ! .. match longest first */
 	match_op(OP_UNKN,UNKN)
 	match_op(OP_UN,UN)
 	match_op(OP_NEGINF,NEGINF)
+	match_op(OP_PREV,PREV)
 	match_op(OP_INF,INF)
 	match_op(OP_NOW,NOW)
 	match_op(OP_TIME,TIME)
@@ -835,7 +888,7 @@ data_calc( image_desc_t *im){
 	for(rpi=0;im->gdes[gdi].rpnp[rpi].op != OP_END;rpi++){
 	    if(im->gdes[gdi].rpnp[rpi].op == OP_VARIABLE){
 		long ptr = im->gdes[gdi].rpnp[rpi].ptr;
-		if ((steparray = rrd_realloc(steparray, (++stepcnt+1)*sizeof(double)))==NULL){
+		if ((steparray = rrd_realloc(steparray, (++stepcnt+1)*sizeof(*steparray)))==NULL){
 		  rrd_set_error("realloc steparray");
 		  free(stack);
 		  return -1;
@@ -897,7 +950,7 @@ data_calc( image_desc_t *im){
 	    for (rpi=0;im->gdes[gdi].rpnp[rpi].op != OP_END;rpi++){
 		if (stptr +5 > dc_stacksize){
 		    dc_stacksize += dc_stackblock;		
-		    stack = rrd_realloc(stack,dc_stacksize*sizeof(long));
+		    stack = rrd_realloc(stack,dc_stacksize*sizeof(*stack));
 		    if (stack==NULL){
 			rrd_set_error("RPN stack overflow");
 			return -1;
@@ -917,6 +970,13 @@ data_calc( image_desc_t *im){
 		    }
 		    stack[++stptr] =  *im->gdes[gdi].rpnp[rpi].data;
 		    break;
+		case OP_PREV:
+		    if (dataidx <= 0) {
+                       stack[++stptr] = DNAN;
+                    } else {
+                       stack[++stptr] = im->gdes[gdi].data[dataidx];
+                    }
+		    break;
 		case OP_UNKN:
 		    stack[++stptr] = DNAN; 
 		    break;
@@ -993,6 +1053,22 @@ data_calc( image_desc_t *im){
 		    }
 		    stack[stptr] = cos(stack[stptr]);
 		    break;
+		case OP_CEIL:
+		    if(stptr<0){
+			rrd_set_error("RPN stack underflow");
+			free(stack);
+			return -1;
+		    }
+		    stack[stptr] = ceil(stack[stptr]);
+		    break;
+		case OP_FLOOR:
+		    if(stptr<0){
+			rrd_set_error("RPN stack underflow");
+			free(stack);
+			return -1;
+		    }
+		    stack[stptr] = floor(stack[stptr]);
+		    break;
 		case OP_LOG:
 		    if(stptr<0){
 			rrd_set_error("RPN stack underflow");
@@ -1044,7 +1120,12 @@ data_calc( image_desc_t *im){
 			free(stack);
 			return -1;
 		    }
-		    stack[stptr-1] = stack[stptr-1] < stack[stptr] ? 1.0 : 0.0;
+		    if (isnan(stack[stptr-1])) 
+			;
+		    else if (isnan(stack[stptr]))
+		        stack[stptr-1] = stack[stptr];
+		    else
+			    stack[stptr-1] = stack[stptr-1] < stack[stptr] ? 1.0 : 0.0;
 		    stptr--;
 		    break;
 		case OP_LE:
@@ -1053,7 +1134,12 @@ data_calc( image_desc_t *im){
 			free(stack);
 			return -1;
 		    }
-		    stack[stptr-1] = stack[stptr-1] <= stack[stptr] ? 1.0 : 0.0;
+		    if (isnan(stack[stptr-1])) 
+			;
+		    else if (isnan(stack[stptr]))
+		        stack[stptr-1] = stack[stptr];
+		    else
+		        stack[stptr-1] = stack[stptr-1] <= stack[stptr] ? 1.0 : 0.0;
 		    stptr--;
 		    break;
 		case OP_GT:
@@ -1062,7 +1148,12 @@ data_calc( image_desc_t *im){
 			free(stack);
 			return -1;
 		    }
-		    stack[stptr-1] = stack[stptr-1] > stack[stptr] ? 1.0 : 0.0;
+		    if (isnan(stack[stptr-1])) 
+			;
+		    else if (isnan(stack[stptr]))
+		        stack[stptr-1] = stack[stptr];
+		    else
+		        stack[stptr-1] = stack[stptr-1] > stack[stptr] ? 1.0 : 0.0;
 		    stptr--;
 		    break;
 		case OP_GE:
@@ -1071,7 +1162,12 @@ data_calc( image_desc_t *im){
 			free(stack);
 			return -1;
 		    }
-		    stack[stptr-1] = stack[stptr-1] >= stack[stptr] ? 1.0 : 0.0;
+		    if (isnan(stack[stptr-1])) 
+			;
+		    else if (isnan(stack[stptr]))
+		        stack[stptr-1] = stack[stptr];
+		    else
+		        stack[stptr-1] = stack[stptr-1] >= stack[stptr] ? 1.0 : 0.0;
 		    stptr--;
 		    break;
 		case OP_EQ:
@@ -1080,7 +1176,12 @@ data_calc( image_desc_t *im){
 			free(stack);
 			return -1;
 		    }
-		    stack[stptr-1] = stack[stptr-1] == stack[stptr] ? 1.0 : 0.0;
+		    if (isnan(stack[stptr-1])) 
+			;
+		    else if (isnan(stack[stptr]))
+		        stack[stptr-1] = stack[stptr];
+		    else
+		        stack[stptr-1] = stack[stptr-1] == stack[stptr] ? 1.0 : 0.0;
 		    stptr--;
 		    break;
 		case OP_IF:
@@ -1093,6 +1194,52 @@ data_calc( image_desc_t *im){
 		    stptr--;
 		    stptr--;
 		    break;
+		case OP_MIN:
+		    if(stptr<1){
+			rrd_set_error("RPN stack underflow");
+			free(stack);
+			return -1;
+		    }
+		    if (isnan(stack[stptr-1])) 
+			;
+		    else if (isnan(stack[stptr]))
+		        stack[stptr-1] = stack[stptr];
+		    else if (stack[stptr-1] > stack[stptr])
+		        stack[stptr-1] = stack[stptr];
+		    stptr--;
+		    break;
+		case OP_MAX:
+		    if(stptr<1){
+			rrd_set_error("RPN stack underflow");
+			free(stack);
+			return -1;
+		    }
+		    if (isnan(stack[stptr-1])) 
+			;
+		    else if (isnan(stack[stptr]))
+		        stack[stptr-1] = stack[stptr];
+		    else if (stack[stptr-1] < stack[stptr])
+		        stack[stptr-1] = stack[stptr];
+		    stptr--;
+		    break;
+		case OP_LIMIT:
+		    if(stptr<2){
+			rrd_set_error("RPN stack underflow");
+			free(stack);
+			return -1;
+		    }
+		    if (isnan(stack[stptr-2])) 
+			;
+		    else if (isnan(stack[stptr-1]))
+		        stack[stptr-2] = stack[stptr-1];
+		    else if (isnan(stack[stptr]))
+		        stack[stptr-2] = stack[stptr];
+		    else if (stack[stptr-2] < stack[stptr-1])
+		        stack[stptr-2] = DNAN;
+		    else if (stack[stptr-2] > stack[stptr])
+		        stack[stptr-2] = DNAN;
+		    stptr-=2;
+		    break;
 		case OP_UN:
 		    if(stptr<0){
 			rrd_set_error("RPN stack underflow");
@@ -1165,7 +1312,7 @@ data_proc( image_desc_t *im ){
 
 		value =
 		    im->gdes[vidx].data[
-					((unsigned long)floor(
+					((unsigned long)floor((double)
 							     (gr_time - im->gdes[vidx].start ) 
 							     / im->gdes[vidx].step)+1)			
 
@@ -1178,16 +1325,15 @@ data_proc( image_desc_t *im ){
 		if (! isnan(value)) {
 		  paintval += value;
 		  im->gdes[ii].p_data[i] = paintval;
+		  if (finite(paintval)){
+  		   if (isnan(minval) || paintval <  minval)
+		     minval = paintval;
+		   if (isnan(maxval) || paintval >  maxval)
+		     maxval = paintval;
+		  }
 		} else {
 		  im->gdes[ii].p_data[i] = DNAN;
 		}
-
-		if (finite(paintval)){
-		  if (isnan(minval) || paintval <  minval)
-		    minval = paintval;
-		  if (isnan(maxval) || paintval >  maxval)
-		    maxval = paintval;
-		}
 		break;
 	    case GF_PRINT:
 	    case GF_GPRINT:
@@ -1271,8 +1417,9 @@ find_first_time(
 	tm.tm_sec=0;
 	tm.tm_min = 0;
 	tm.tm_hour = 0;
-	tm.tm_mday -= tm.tm_wday -1 ; break;	/* -1 because we want the monday */
+	tm.tm_mday -= tm.tm_wday -1;	/* -1 because we want the monday */
 	if (tm.tm_wday==0) tm.tm_mday -= 7; /* we want the *previous* monday */
+	break;
     case TMT_MONTH:
 	tm.tm_sec=0;
 	tm.tm_min = 0;
@@ -1444,6 +1591,10 @@ print_calc(image_desc_t *im, char ***prdata)
 	    }
 	    if (im->gdes[i].gf == GF_PRINT){
 		(*prdata)[prlines-2] = malloc((FMT_LEG_LEN+2)*sizeof(char));
+		if (bad_format(im->gdes[i].format)) {
+			rrd_set_error("bad format for [G]PRINT in '%s'", im->gdes[i].format);
+			return -1;
+		}
 #ifdef HAVE_SNPRINTF
 		snprintf((*prdata)[prlines-2],FMT_LEG_LEN,im->gdes[i].format,printval,si_symb);
 #else
@@ -1453,6 +1604,10 @@ print_calc(image_desc_t *im, char ***prdata)
 	    } else {
 		/* GF_GPRINT */
 
+		if (bad_format(im->gdes[i].format)) {
+			rrd_set_error("bad format for [G]PRINT in '%s'", im->gdes[i].format);
+			return -1;
+		}
 #ifdef HAVE_SNPRINTF
 		snprintf(im->gdes[i].legend,FMT_LEG_LEN-2,im->gdes[i].format,printval,si_symb);
 #else
@@ -1481,7 +1636,7 @@ print_calc(image_desc_t *im, char ***prdata)
 
 
 /* place legends with color spots */
-void
+int
 leg_place(image_desc_t *im)
 {
     
@@ -1496,6 +1651,12 @@ leg_place(image_desc_t *im)
     int   glue = 0;
     int   i,ii, mark = 0;
     char  prt_fctn; /*special printfunctions */
+    int  *legspace;
+
+    if ((legspace = malloc(im->gdes_c*sizeof(int)))==NULL){
+       rrd_set_error("malloc for legspace");
+       return -1;
+    }
 
     for(i=0;i<im->gdes_c;i++){
 	fill_last = fill;
@@ -1505,22 +1666,38 @@ leg_place(image_desc_t *im)
 	/* is there a controle code ant the end of the legend string ? */ 
 	if (leg_cc >= 2 && im->gdes[i].legend[leg_cc-2] == '\\') {
 	    prt_fctn = im->gdes[i].legend[leg_cc-1];
-	    im->gdes[i].legend[leg_cc-2] = '\0';
 	    leg_cc -= 2;
+	    im->gdes[i].legend[leg_cc] = '\0';
 	} else {
 	    prt_fctn = '\0';
 	}
-
+        /* remove exess space */
+        while (prt_fctn=='g' && 
+	       leg_cc > 0 && 
+	       im->gdes[i].legend[leg_cc-1]==' '){
+	   leg_cc--;
+	   im->gdes[i].legend[leg_cc]='\0';
+	}
 	if (leg_cc != 0 ){	    
-	    if (fill > 0) 
-		fill += interleg;
+	   legspace[i]=(prt_fctn=='g' ? 0 : interleg);
+	   
+	   if (fill > 0){ 
+ 	       /* no interleg space if string ends in \g */
+	       fill += legspace[i];
+	    }
 	    if (im->gdes[i].gf != GF_GPRINT && 
-		im->gdes[i].gf != GF_COMMENT) 
+		im->gdes[i].gf != GF_COMMENT) { 
 		fill += box; 	   
+	    }
 	    fill += leg_cc * SmallFont->w;
 	    leg_c++;
+	} else {
+	   legspace[i]=0;
+	}
+        /* who said there was a special tag ... ?*/
+	if (prt_fctn=='g') {    
+	   prt_fctn = '\0';
 	}
-	    
 	if (prt_fctn == '\0') {
 	    if (i == im->gdes_c -1 ) prt_fctn ='l';
 	    
@@ -1558,7 +1735,7 @@ leg_place(image_desc_t *im)
 		im->gdes[ii].legloc.y = leg_y;
 		leg_x =  leg_x 
 		    + strlen(im->gdes[ii].legend)*SmallFont->w 
-		    + interleg 
+		    + legspace[ii]
 		    + glue;
 		if (im->gdes[ii].gf != GF_GPRINT && 
 		    im->gdes[ii].gf != GF_COMMENT) 
@@ -1572,6 +1749,8 @@ leg_place(image_desc_t *im)
 	}	   
     }
     im->ygif = leg_y+6;
+    free(legspace);
+    return 0;
 }
 
 
@@ -1623,7 +1802,7 @@ horizontal_grid(gdImagePtr gif, image_desc_t   *im)
 		sprintf(labfmt, "%%%d.%df", decimals - fractionals + 1, -fractionals + 1);
 	    else
 		sprintf(labfmt, "%%%d.1f", decimals + 1);
-	    gridstep = pow(10, fractionals);
+	    gridstep = pow((double)10, (double)fractionals);
 	    if(gridstep == 0) /* range is one -> 0.1 is reasonable scale */
 		gridstep = 0.1;
 	    /* should have at least 5 lines but no more then 15 */
@@ -1697,7 +1876,7 @@ horizontal_grid(gdImagePtr gif, image_desc_t   *im)
                               (polyPoints[0].x - (strlen(graph_label) * 
                                                   SmallFont->w)-7), 
                               polyPoints[0].y - SmallFont->h/2+1,
-                              graph_label, graph_col[GRC_FONT].i);
+                              (unsigned char *)graph_label, graph_col[GRC_FONT].i);
 		
 		gdImageSetStyle(gif, styleMajor, 2);
 
@@ -1766,7 +1945,7 @@ horizontal_log_grid(gdImagePtr gif, image_desc_t   *im)
     polyPoints[0].x=im->xorigin;
     polyPoints[1].x=im->xorigin+im->xsize;
     /* paint minor grid */
-    for (value = pow(10, log10(im->minval) 
+    for (value = pow((double)10, log10(im->minval) 
 			  - fmod(log10(im->minval),log10(yloglab[minoridx][0])));
 	 value  <= im->maxval;
 	 value *= yloglab[minoridx][0]){
@@ -1787,7 +1966,7 @@ horizontal_log_grid(gdImagePtr gif, image_desc_t   *im)
     }
 
     /* paint major grid and labels*/
-    for (value = pow(10, log10(im->minval) 
+    for (value = pow((double)10, log10(im->minval) 
 			  - fmod(log10(im->minval),log10(yloglab[majoridx][0])));
 	 value <= im->maxval;
 	 value *= yloglab[majoridx][0]){
@@ -1809,7 +1988,7 @@ horizontal_log_grid(gdImagePtr gif, image_desc_t   *im)
 			  (polyPoints[0].x - (strlen(graph_label) * 
 					      SmallFont->w)-7), 
 			  polyPoints[0].y - SmallFont->h/2+1,
-			  graph_label, graph_col[GRC_FONT].i);	
+			  (unsigned char *)graph_label, graph_col[GRC_FONT].i);	
 	} 
     }
 	return 1;
@@ -1919,7 +2098,7 @@ vertical_grid(
 	    && gr_pos + width <= im->xorigin+im->xsize) 
 	    gdImageString(gif, SmallFont,
 			  gr_pos,  polyPoints[0].y+4,
-			  graph_label, graph_col[GRC_FONT].i);
+			  (unsigned char *)graph_label, graph_col[GRC_FONT].i);
     }
 
 }
@@ -2001,7 +2180,7 @@ grid_paint(
 		     im->xgif/2 
 		     - (strlen(nodata)*LargeFont->w)/2,
 		     (2*im->yorigin-im->ysize) / 2,
-		     nodata, graph_col[GRC_FONT].i);
+		     (unsigned char *)nodata, graph_col[GRC_FONT].i);
     }
 
 
@@ -2010,7 +2189,7 @@ grid_paint(
 		    7,
 		    (im->yorigin - im->ysize/2
 		     +(strlen(im->ylegend)*SmallFont->w)/2 ),
-		    im->ylegend, graph_col[GRC_FONT].i);
+		    (unsigned char *)im->ylegend, graph_col[GRC_FONT].i);
     
 
     /* graph title */
@@ -2018,7 +2197,7 @@ grid_paint(
 		    im->xgif/2 
 		    - (strlen(im->title)*LargeFont->w)/2,
 		  8,
-		    im->title, graph_col[GRC_FONT].i);
+		    (unsigned char *)im->title, graph_col[GRC_FONT].i);
     
     /* graph labels */
 
@@ -2042,7 +2221,7 @@ grid_paint(
 	    gdImageString(gif, SmallFont,
 			  polyPoints[0].x+boxH+6, 
 			  polyPoints[0].y-1,
-			  im->gdes[i].legend,
+			  (unsigned char *)im->gdes[i].legend,
 			  graph_col[GRC_FONT].i);
 	} else {
 	    polyPoints[0].x = im->gdes[i].legloc.x;
@@ -2051,7 +2230,7 @@ grid_paint(
 	    gdImageString(gif, SmallFont,
 			  polyPoints[0].x, 
 			  polyPoints[0].y,
-			  im->gdes[i].legend,
+			  (unsigned char *)im->gdes[i].legend,
 			  graph_col[GRC_FONT].i);
 	}
     }
@@ -2116,6 +2295,7 @@ MkLineBrush(image_desc_t *im,long cosel, enum gf_en typsel){
 
 int lazy_check(image_desc_t *im){
     FILE *fd = NULL;
+    int size = 1;
     struct stat  gifstat;
     
     if (im->lazy == 0) return 0; /* no lazy option */
@@ -2130,14 +2310,14 @@ int lazy_check(image_desc_t *im){
       return 0; /* the file does not exist */
     switch (im->imgformat) {
     case IF_GIF:
-	GifSize(fd,&(im->xgif),&(im->ygif));
+	size = GifSize(fd,&(im->xgif),&(im->ygif));
 	break;
     case IF_PNG:
-	PngSize(fd,&(im->xgif),&(im->ygif));
+	size = PngSize(fd,&(im->xgif),&(im->ygif));
 	break;
     }
     fclose(fd);
-    return 1;
+    return size;
 }
 
 /* draw that picture thing ... */
@@ -2206,7 +2386,8 @@ graph_paint(image_desc_t *im, char ***calcpr)
     
     /* determine where to place the legends onto the graphics.
        and set im->ygif to match space requirements for text */
-    leg_place(im);
+    if(leg_place(im)==-1)
+     return -1;
 
     gif=gdImageCreate(im->xgif,im->ygif);
 
@@ -2384,7 +2565,7 @@ graph_paint(image_desc_t *im, char ***calcpr)
 	fo = stdout;
     } else {
 	if ((fo = fopen(im->graphfile,"wb")) == NULL) {
-	    rrd_set_error("cannot open %s for write",im->graphfile);
+	    rrd_set_error("Openin %s for write: %s",im->graphfile, strerror(errno));
 	    return (-1);
 	}
     }
@@ -2535,6 +2716,7 @@ rrd_graph(int argc, char **argv, char ***prdata, int *xsize, int *ysize)
 	    {"lazy",       no_argument,       0,  'z'},
 	    {"alt-y-grid", no_argument,       0,   257 },
 	    {"alt-autoscale", no_argument,    0,   258 },
+	    {"alt-autoscale-max", no_argument,    0,   259 },
 	    {0,0,0,0}};
 	int option_index = 0;
 	int opt;
@@ -2554,6 +2736,9 @@ rrd_graph(int argc, char **argv, char ***prdata, int *xsize, int *ysize)
 	case 258:
 	    im.extra_flags |= ALTAUTOSCALE;
 	    break;
+	case 259:
+	    im.extra_flags |= ALTAUTOSCALE_MAX;
+	    break;
 	case 's':
 	    if ((parsetime_error = parsetime(optarg, &start_tv))) {
 	        rrd_set_error( "start time: %s", parsetime_error );
@@ -2690,19 +2875,26 @@ rrd_graph(int argc, char **argv, char ***prdata, int *xsize, int *ysize)
 	    break;
 
 	case '?':
-	    rrd_set_error("unknown option '%s'",argv[optind-1]);
-	    return -1;
+            if (optopt != 0)
+                rrd_set_error("unknown option '%c'", optopt);
+            else
+                rrd_set_error("unknown option '%s'",argv[optind-1]);
+            return -1;
 	}
     }
     
+    if (optind >= argc) {
+       rrd_set_error("missing filename");
+       return -1;
+    }
 
     if (im.logarithmic == 1 && (im.minval <= 0 || isnan(im.minval))){
 	rrd_set_error("for a logarithmic yaxis you must specify a lower-limit > 0");	
 	return -1;
     }
 
-    strncpy(im.graphfile,argv[optind],254);
-    im.graphfile[254]='\0';
+    strncpy(im.graphfile,argv[optind],MAXPATH-1);
+    im.graphfile[MAXPATH-1]='\0';
 
     if (proc_start_end(&start_tv,&end_tv,&start_tmp,&end_tmp) == -1){
 	return -1;
@@ -2903,7 +3095,7 @@ rrd_graph(int argc, char **argv, char ***prdata, int *xsize, int *ysize)
 		if(sscanf(&argv[i][argstart
 				  +strstart
 				  +scan_for_col(&argv[i][argstart+strstart],
-						254,im.gdes[im.gdes_c-1].rrd)],
+						MAXPATH,im.gdes[im.gdes_c-1].rrd)],
 			  ":" DS_NAM_FMT ":" CF_NAM_FMT,
 			  im.gdes[im.gdes_c-1].ds_nam,
 			  symname) != 2){
@@ -2974,4 +3166,30 @@ rrd_graph(int argc, char **argv, char ***prdata, int *xsize, int *ysize)
     return 0;
 }
 
+int bad_format(char *fmt) {
+	char *ptr;
+
+	ptr = fmt;
+	while (*ptr != '\0') {
+		if (*ptr == '%') {ptr++;
+			if (*ptr == '\0') return 1;
+			while ((*ptr >= '0' && *ptr <= '9') || *ptr == '.') { 
+				ptr++;
+			}
+			if (*ptr == '\0') return 1;
+			if (*ptr == 'l') {
+				ptr++;
+				if (*ptr == '\0') return 1;
+				if (*ptr == 'e' || *ptr == 'f') { 
+					ptr++; 
+					} else { return 1; }
+			}
+			else if (*ptr == 's' || *ptr == 'S' || *ptr == '%') { ++ptr; }
+			else { return 1; }
+		} else {
+			++ptr;
+		}
+	}
+	return 0;
+}
 
